!============================================================================*
!                                                                            *
!  lrschema.spc LIBERO code generation schema for the state pattern in C++   *
!                                                                            *
!                                                                            *
!  YOU NEED AT LEAST LIBERO 2.30!                                            *
!                                                                            *
!                                                                            *
!  Written:    1997-06-10  Mathias Weidner <mathias@weidner.sem.lipsia.de>   *
!                                                                            *
!  Thanks to:  Pieter Hintjens <ph@imatix.com> for LIBERO                    *
!                                                                            *
!  Usage:      Generates these files:                                        *
!              ${source}i.h    Interface for state and actor class           *
!              ${source}s.h    Interface for concrete state classes          *
!              ${source}s.cpp  Definitions of concrete state classes         *
!                                                                            *
!               Accepts these options:                                       *
!               -opt:template=xxxx - template file (template.cpp)            *
!               -opt:prefix=xxxx   - class name (C$source)                   *
!                                                                            *
!   LIBERO Copyright (c) 1991-97 iMatix.                                     *
!                                                                            *
!   This schema is free software; you can redistribute it and/or modify      *
!   it under the terms of the GNU General Public License as published by     *
!   the Free Software Foundation; either version 2 of the License, or        *
!   (at your option) any later version.                                      *
!                                                                            *
!   This program is distributed in the hope that it will be useful,          *
!   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
!   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
!   GNU General Public License for more details.                             *
!                                                                            *
!   You should have received a copy of the GNU General Public License        *
!   along with this program; if not, write to the Free Software              *
!   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                *
!============================================================================*

!   You can use the template file to include project-specific code in the
!   generated code at various points.  This is a (better) alternative to
!   changing the schema in order to add your own project code.
!
!   The template file should contain these sections (each starts with a
!   keyword on a line, and ends with <END> on a line).  These sections may
!   be included in newly-generated code (keywords are case-sensitive):
!
!   <HEADER>                    - program header comment
!   <HEADER_DECLARATIONS>       - additional includes, declarations
!                                 and definitions in $(source)i.h

!:if $version < 230
: echo Remember! You need at least libero 2.30.
!: exit 1
!:endif
:declare string $template     = "template.cpp"
:declare string $prefix       = "C$source"
:declare string $state_name   = ""
:declare string $default_name = ""

:echo Generating $source\i.h ...
:output $source\i.h
//====================================================================
// $source\i.h - Interface declarations for state pattern classes
//
// Written:    $date    $author
// Revised:    $date
//
:include optional $template "<HEADER>" "<END>"
// This file was generated by LIBERO $version on $fulldate, $time
//====================================================================
#ifndef __$Source\I__INCLUDED
#define __$Source\I__INCLUDED

:include optional $template "<HEADER_DECLARATIONS>" "<END>"

//--------------------------------------------------------------------
// This needs to be forward declared
//
class $prefix\Context;

//--------------------------------------------------------------------
class $prefix\State {
public:

  virtual ~$prefix\State() = 0;

:do event
  virtual void $name($prefix\Context &);

:enddo
  virtual char * Name();

  static $prefix\State * InitialState();

}; // class $prefix\State

inline $prefix\State::~$prefix\State() {}

//--------------------------------------------------------------------
class $prefix\Context {
public:

  $prefix\Context($prefix\State * = $prefix\State::InitialState());
  virtual ~$prefix\Context();

:do module
  virtual void $name();

:enddo
  virtual void UnexpectedEvent(const char * state,
                               const char * event);

  void SetState($prefix\State * s);
  $prefix\State * GetState();

private:

  $prefix\State * state;

}; // class $prefix\Context

inline $prefix\Context::$prefix\Context($prefix\State * s) : state(s) {}
inline $prefix\Context::~$prefix\Context() {}
inline void $prefix\Context::SetState($prefix\State * s){state = s;}
inline $prefix\State * $prefix\Context::GetState(){return state;}

#endif
:close $source\i.h
:echo Generating $source\s.h ...
:output $source\s.h
//====================================================================
// $source\s.h - Declarations for concrete state classes
//
// Written:    $date    $author
// Revised:    $date
//
:include optional $template "<HEADER>" "<END>"
// This file was generated by LIBERO $version on $fulldate, $time
//====================================================================
#ifndef __$Source\S__INCLUDED
#define __$Source\S__INCLUDED

#include "$source\i.h"

:set $default_name = $prefix\State
:if defaults
: do state
:  set $state_name = $name
:  if $number == $defaults
class $prefix\$name
 : public $default_name
{
public:
: do event local
  virtual void $name($prefix\Context &);

: enddo
  virtual char * Name();

  static $prefix\State * GetInstance();

}; // class $prefix\$name

:   set $default_name = $prefix$state_name
:  endif
: enddo
: do state
:  set $state_name = $name
:  if $number != $defaults
class $prefix\$name
 : public $default_name
{
public:
: do event local
  virtual void $name($prefix\Context &);

: enddo
  virtual char * Name();

  static $prefix\State * GetInstance();

}; // class $prefix\$name

:  endif
: enddo
:else
: do state
:  set $state_name = $name
class $prefix\$name
 : public $default_name
{
public:
: do event local
  virtual void $name($prefix\Context &);

: enddo
  virtual char * Name();

  static $prefix\State * GetInstance();

}; // class $prefix\$name

: enddo
:endif

#endif
:close $source\s.h
:echo Generating $source\s.cpp ...
:output $source\s.cpp
//====================================================================
// $source\s.cpp - Implementation of concrete state classes
//
// Written:    $date    $author
// Revised:    $date
//
:include optional $template "<HEADER>" "<END>"
// This file was generated by LIBERO $version on $fulldate, $time
//====================================================================

#include "$source\s.h"

// -------------------------------------------------------------------
// Default functions for all events
//
:do event
void $prefix\State::$name($prefix\Context & doForMe)
{
  doForMe.UnexpectedEvent(Name(),"$name");
}

:enddo

char * $prefix\State::Name()
{
  return "$prefix\State";
}

:do state
: set $state_name = $name
// -------------------------------------------------------------------
// $prefix$state_name
//
: if $number == 0
$prefix\State * $prefix\State::InitialState()
{
  return $prefix$state_name::GetInstance();
}

: endif
$prefix\State * $prefix$state_name::GetInstance()
{
  static $prefix$state_name instance;

  return & instance;
}

char * $prefix$state_name::Name()
{
  return "$prefix$state_name";
}

: do event local
void $prefix$state_name::$name($prefix\Context & doForMe)
{
:  do module local
   doForMe.$name();
:  enddo
:  if "$state_name" != "$next_state"
   doForMe.SetState($prefix$next_state::GetInstance());
:  endif

} // $prefix$state_name::$name()

: enddo

:enddo
:close $source\s.cpp
